/*
	PROforma, PROgs FOnt & Raster MAnager

	Get information about the available drivers
	PFDriverCount
	PFDriverNext

	PFPictureCount
	PFPictureNext

	(c) PROGS, Professional & Graphical Software
	written by Joachim Van der Auwera
	started 26 July 1993
	28.10.1995 - with new driver id's in PROforma v1.10 onwards, the id was
			   wrongly initialised and incremented in PFDriverCount/Next
			   - NULL can now be passed if result not wanted
			   - PFDriverNext returns ERR_IPAR when end of list reached
	09.11.1995 the error would better be ERR_EOF
	18.12.1995 added functions to scan through picture drivers
	14.07.1996 counting when no drivers should now also work
*/

#include "gstate.h"

extern DRIVdriver *printers;	/* currently supported printer drivers */
extern PICTdriver *pictures;	/* currently supported picture drivers */
extern BITMdriver *bitmaps; 	/* currently supported bitmap drivers */


/* enquire number of available drivers, and make sure PFDriverNext will return
   the first one the next time */
Error PFDriverCount(Gstate gstate, int *count)
{
	DRIVdriver *driv=printers;
	int _count;

	if (count==NULL) count=&_count;

	*count=0;
	while (driv && driv->next) (*count)++, driv=driv->next;
	gstate->nextdriver=printers;
	gstate->nextdrivernumber=-2;
	return ERR_OK;
}

/* get id and name of next device */
Error PFDriverNext(Gstate gstate, int *id, char *name)
{
	DRIVdriver *d=gstate->nextdriver;

	if (!d) return ERR_EOF;

	if (id) *id=gstate->nextdrivernumber;
	if (name) MEMCopy(PF_MAXDRIVERNAME,d->name,name);

	gstate->nextdrivernumber--;
	gstate->nextdriver=d->next;

	return ERR_OK;
}

Error PFPictureCount(Gstate gstate, int *count)
{
	PICTdriver *driv=pictures;
	int _count;

	if (count==NULL) count=&_count;

	*count=0;
	while (driv && driv->next) (*count)++, driv=driv->next;
	gstate->nextpicture=pictures;
	gstate->nextpicturenumber=-1;
	return ERR_OK;
}

Error PFPictureNext(Gstate gstate, int *id, char *name)
{
	PICTdriver *d=gstate->nextpicture;

	if (!d) return ERR_EOF;

	if (id) *id=gstate->nextpicturenumber;
	if (name) MEMCopy(PF_MAXDRIVERNAME,d->name,name);

	gstate->nextpicturenumber--;
	gstate->nextpicture=d->next;

	return ERR_OK;
}
