/*
	PROforma, PROgs FOnt & Raster MAnager

	Fill routine
		special version for drawing characters
		this version does not fill using trapezoids, but using horizontal
		lines only. This reduces the complexity a bit, and as characters are
		quite complex and use accurate curves, changes are small that real
		trapezoids are ever used.
		This version of fillpath uses no clipping path, and always uses the
		winding rule.
		The error code is always ERR_OK

	(c) PROGS, Professional & Graphical Software
	written by Joachim Van der Auwera
	30.11.1996 started based on normal fillpath
*/

#include "gstate.h"
#include "system.h"
#include "path.h"

/* forward declarations */
static segment *add_xlist(segment *ylist, segment *xlist);

#define nexthxpos(a) (a->startx/*+(a->incr/2)*/)
#define nextxpos(a) (a->startx+a->incr)

Error cfillpath(Gstate gstate,
				Error (*Line)(Bitmap, short, short, short),
				Bitmap bitmap)
{
	segment *sh, *ss;
	segment *xlist, *ylist; 		/* xlist, last element in xlist, ylist */
	segment *el, *next; 			/* used for running through lists */
	char done=FALSE;				/* TRUE when lists are empty */
	short starty, endy; 			/* band boundaries */
	short wind; 					/* counter to determine in or outside */
	short leftside; 				/* TRUE if leftside is initialised */
	pt leftstartx;					/* position for leftside of line */

	/* remove the header and sentinel segments, and use to initialise the */
	/* xlist, we only need the linkage with next in the ylist */
	xlist=sh=gstate->path;
	ss=sh->prev;
	ylist=sh->next;
	if (ylist==ss) return ERR_OK;	/* there has to be something to do */
	sh->next=ss; sh->prev=NULL; 	/* has to be zero for calc_nexty to work */
	ss->prev->next=NULL;
	ss->prev=sh;

	/* start building xlist by filling in all segment with same starty */
	endy=ylist->starty; 			/* start of band */
	ylist=add_xlist(ylist, xlist);

	/* process all the bands inside PageBbox */
	while(!done)
	{
		starty=endy;				/* initialise this band */
		endy=starty+1;
		wind=0; leftside=FALSE;

		/* now run through the xlist */
		el=xlist->next; 			/* skip header */
		while (el->next)
		{
			next=el->next;			/* keep next in case of removal */

			/* adjust counter for window rule */
			if (!el->isclip) wind+=el->direction;

			/* is this a leftside ?? */
			if (wind)
			{
				/* is this a leftside ?? */
				if (!leftside && wind)
				{
					leftside=TRUE;
					leftstartx=nexthxpos(el);
				}
			}
			else
			{
				/* is this a rightside ?? */
				if (leftside)
				{
					short  left=pt2rshort(leftstartx);
					short right=pt2rshort(nexthxpos(el));
					if (left==right)
					{
						left=pt2short(leftstartx);
						right=left+1;
					}
					Line(bitmap, starty, left, right);
					leftside=FALSE;
				}
			}
			/* now reduce band from this element */
			el->starty=endy;
			el->startx+=el->incr;
			if (endy>=el->endy)
			{
				/* unlink and free element from xlist */
				el->next->prev=el->prev;
				el->prev->next=el->next;
				free_segment(gstate,el);
			} else {
				/* re-order this element in xlist */
				/* this involves recalculating all elements with a new prev */

				if ( nextxpos(el->prev) > nextxpos(el) )
				{
					segment *s=el, *list=el->next;
					pt x=nexthxpos(s);

					/* linkout first */
					s->next->prev=s->prev;
					s->prev->next=s->next;
					el=s->prev; 	/* make sure continue with correct segment */

					/* relink in place now */
					while (nexthxpos(list->prev)>x) list=list->prev;
					/* link in before segment *list */
					s->prev=list->prev;
					s->next=list;
					s->prev->next=s;
					list->prev=s;
				}
			}
			el=next;
		}

		if (!ylist && !xlist->next->next) done=TRUE;	/* lists empty->stop */

		if (ylist && endy==ylist->starty) ylist=add_xlist(ylist, xlist);
	}

	/* now clean up some internal data */
	gstate->path=xlist; 	  /* set path to be released, not valid !! */
	return ERR_OK;
}

static segment *add_xlist(segment *ylist, segment *xlist)
{
	short y;
	segment *s;
	pt x;

	if (ylist) y=ylist->starty;
	while(ylist && ylist->starty==y)
	{
		linkout(s,ylist);
		/* now link s in xlist */
		x=nexthxpos(s);
		/* find item before which we have to link in */
		while (nexthxpos(xlist)>x) xlist=xlist->prev;
		while (nexthxpos(xlist)<x) xlist=xlist->next;
		/* link in before segment *xlist */
		s->prev=xlist->prev;
		s->next=xlist;
		s->prev->next=s;
		xlist->prev=s;
	}
	return ylist;
}
