/*
	PROforma, PROGS FOnt & Raster MAnager

	Printer drivers for all Epson ESC\P2 (compatible) printers

	(c) PROGS, Professional & Graphical Software
	written by Joachim Van der Auwera
	started 30 July 1994

	this file is based on PROforma v1.0? DeskJet external driver
	02.11.1995 for PROforma v1.10 - different driver structure
	02.11.1995 finally introduced skip_lines in driver - better speed etc.
	29.06.1996 fixed a bug in CompressMode2
*/

#include "io.h"
#include "driv/escp2.h"

/* this driver allows both 180 and 360dpi resolution, */
/* x and y resolution must be the same */

/* forward references */
static int CompressMode2(char *, char *, char *);

/* some macros for generating output */
#define printb(b,s) IOPutRawBytes(chid,s,(char *)b,NULL)
#define prints(s)	printb(s,strlen(s))
#define printc(c)	{ char s=c; IOPutRawBytes(chid,1,&s,NULL); }

/* initialise the printer */

Error InitPage(Gstate gstate)
{
	DriverInfo *di=gstate->PrinterExtra;
	Channel chid=di->channel;

	/* reset printer */
	prints("\033@");

	/* set graphics mode */
	prints("\x1b(G\1");
	printc(0); printc(1);

	if (gstate->printer->xdpi==180)
	{
		prints("\x1b(U\1");
		printc(0); printc(20);		/* 180 dpi resolution */
		prints("\x1b+\2");			/* and line spacing */
	} else {
		prints("\x1b(U\1");
		printc(0); printc(10);		/* 360 dpi resolution */
		prints("\x1b+\1");			/* and line spacing */
	}

	prints("\x1b(V\02");			/* set absolute vertical position */
	printc('\0'); printc(di->yskip&0xff); printc(di->yskip>>8);
	di->skip_lines=0;
	return ERR_OK;
}

/* Send buffer to printer - use compression to speed up data transfer */

Error ShowPage(Gstate gstate)
{
	DriverInfo *di=gstate->PrinterExtra;
	Channel chid=di->channel;
	int res=gstate->printer->xdpi;
	int line_size=gstate->raster->linc;
	char *data=(char *)gstate->raster->base;
	char *end_data;
	char *out_data=di->out_line;
	int out_len, i, pixcount;
	int skip_lines=di->skip_lines;

	for (i=gstate->pagebbox.ysiz ; i ; i--, data+=line_size )
	{
		pixcount=line_size*8;
		end_data=data+line_size;

		/* remove trailing zeros - printing "nothing" is useless ! */
		while (end_data>data && !end_data[-1]) end_data--, pixcount-=8;
		if (end_data==data)
		{
			skip_lines++;
			continue;
		}

		if (end_data!=data)
		{
			if (skip_lines)
			{
				prints("\033(v\02"); printc(0);
				printc(skip_lines&0xff); printc(skip_lines>>8);
				skip_lines=0;
			}
			/* compress the line */
			out_len=CompressMode2(data, end_data, out_data);
			/* set horizontal start position */
			if (di->xskip)
			{
				prints("\x1b$");
				printc(di->xskip&0xff); printc(di->xskip>>8);
			}
			/* set raster graphics mode */
			if (res==180) { prints("\033.\1\x14\x14\1"); }
					 else { prints("\033.\1\x0a\x0a\1"); }
			printc(pixcount&0xff);
			printc(pixcount>>8);
			/* send the line and move to next */
			printb(out_data,out_len);
			prints("\r\n");
		}
	}
	di->skip_lines=skip_lines;
	return ERR_OK;
}

/* handles everything to end the printing */

Error StopPage(Gstate gstate)
{
	Channel chid=gstate->PrinterExtra->channel;

	/* eject page and end raster graphics */
	prints("\f\033@");

	return ERR_OK;
}

/* mode 2 compression routine or TIFF (rev. 4.0) encoding */
/* variation on run length encoding */
/* theoretically, the output can be at most twice as long as the input */
/* however - the CompressMode2 routine produces an output with maximum */
/* length size+size/127+1 */

static int CompressMode2(char *line, char *end, char *output)
{
	char *org_output=output;
	char *run_start;
	char previous;
	char identical=FALSE;
	int counter;

	while (line<end)
	{
		run_start=line;
		if (identical)
		{
			previous=*line++;
			while (line<end && *line==previous) line++;
			for (counter=line-run_start ; counter>128 ; counter-=128)
			{
				*output++=-127;
				*output++=previous;
			}
			if (counter>1)
			{
				*output++=-counter+1;
				*output++=previous;
			}
			else
				line--;
			identical=FALSE;
		}
		else
		{
			/* need three identical bytes for compression to stay optimal */
			/* I could also request another identical byte, this would */
			/*	- make longer runs - might be more efficient for the printer */
			/*	  but then again, sending data to the printer is so slow */
			/*	- longer runs makes it easier to get a break because you can */
			/*	  run out of counter bits, therefore, the compression can */
			/*	  be BETTER by only going for three identical bytes */
			while (line+2<end && (line[1]!=line[0] || line[2]!=line[0])) line++;
			if (line+2>=end) line=end;
			for (counter=line-run_start ; counter>128 ; counter-=128)
			{
				*output++=127;
				MEMCopy(128,run_start,output);
				output+=128;
				run_start+=128;
			}
			if (counter)
			{
				*output++=counter-1;
				MEMCopy(counter,run_start,output);
				output+=counter;
			}
			identical=TRUE;
		}
	}
	return output-org_output;
}
