/*
	PROforma, PROGS Font & Raster Manager

	g3 compatible driver for Qfax
	the code to produce the g3 image is (c) Jonathan Hudson

	(c) PROGS, Professional & Graphical Software
	written by Joachim Van der Auwera
	15.03.1997 started based on monopic printer driver
*/

#include <io.h>
#include <mem.h>
#include <win.h>
#include "PFmodule.h"
#include "driv/faxg3encoder.h"

typedef struct _DriverInfo {
	Channel channel;		/* the output channel for this driver */
	CODERSTATE state;		/* state for g3 encoder */
	int pagecount;			/* number of pages in fax */
} DriverInfo;

Bbox PrintableArea={
	short2pt(564), short2pt(822),	/* size */
	short2pt(18), short2pt(18)		/* origin */
};

static Error Define(char *name, char *value)
{
	LinkPROforma();
	return ERR_IPAR;
}

static Error InitInit(Gstate gstate, char *device, PBbox *area)
{
	Error err;
	DriverInfo *di;
	DRIVdriver *prt=gstate->printer;

	err=MEMAllocate(sizeof(DriverInfo),&di);
	if (err) return err;

	if (device && *device)
		catch( IOOpen(device,OPEN_NEW,&di->channel) );
	else
	{
		MEMRelease(di);
		return ERR_ITNF;
	}

	/* initialise qfax blackbox */
	FaxInitCoder(&di->state);
	FaxOpenFile(&di->state,di->channel,(prt->ydpi/98)-1,0,1728);
	FaxHeader(&di->state,0,0,0,NULL);

	gstate->PrinterExtra=di;
	gstate->todev.x = short2pt(prt->xdpi) / 72;
	gstate->todev.y = short2pt(prt->ydpi) / 72;
	gstate->ppp.x = short2pt(72) / prt->xdpi;
	gstate->ppp.y = short2pt(72) / prt->ydpi;

	area->xsiz=pt2rshort(fixmul(PrintableArea.xsiz,gstate->todev.x));
	area->ysiz=pt2rshort(fixmul(PrintableArea.ysiz,gstate->todev.y));
	area->xorg=pt2rshort(fixmul(PrintableArea.xorg,gstate->todev.x));
	area->yorg=pt2rshort(fixmul(PrintableArea.yorg,gstate->todev.y));

	return err;
}

static Error InitDone(Gstate gstate)
{
	return ERR_OK;
}

static Error BufferSize(Gstate gstate, int xsiz, int ysiz, Size *size)
{
	return gstate->bitmap->BufferSize(xsiz,ysiz,size);
}

static Error CleanUp(Gstate gstate)
{
	DriverInfo *di=gstate->PrinterExtra;

	if (di)
	{
		FaxHeader(&di->state, di->pagecount, 0,2, "PROforma fax driver");
		FaxCloseFile(&di->state);
		FaxFreeCoder(&di->state);
		IOClose(di->channel);
		MEMRelease(di);
		gstate->PrinterExtra=NULL;
	}
	return ERR_OK;
}

static Error ColourNearest(Gstate gstate, pt *colour, int *result)
{
	*result=0;						/* assume white */
	if (*colour>pt_hundred/2)
		*result=1;					/* make it black */
	return ERR_OK;
}

static Error ColourDelta(Gstate gstate, pt *colour, pt *delta)
{
	*delta=*colour;
	if (*colour>pt_hundred/2)
		*delta-=pt_hundred;
	return ERR_OK;
}

/* !!!!!!!!!!! code to send data to printer !!!!!!!!!!! */

/* some macros for generating output */
#define printb(s,b) IOPutRawBytes(chid,s,b,NULL)

Error InitPage(Gstate gstate)
{
	DriverInfo *di=gstate->PrinterExtra;
	FaxBeginPage(&di->state);
	return ERR_OK;
}

Error ShowPage(Gstate gstate)
{
	DriverInfo *di=gstate->PrinterExtra;
	int line_size=gstate->raster->linc;
	unsigned char *data=gstate->raster->base;
	short lnum;
	unsigned char line[216];

	for(lnum=gstate->pagebbox.ysiz; lnum; lnum--)
	{
		MEMClear(216,line);
		MEMCopy(200,data,line+8);
		FaxCodeLine(&di->state,line);
		data+=line_size;
	}
	return ERR_OK;
}

Error StopPage(Gstate gstate)
{
	DriverInfo *di=gstate->PrinterExtra;
	FaxCodeEOP(&di->state);
	di->pagecount++;				/* one more page in fax */
	return ERR_OK;
}


/* !!!!!!!!!!! printer driver definition !!!!!!!!!!! */

static Error nimp() 	{ return ERR_NIMP; }
static Error nothing()	{ return ERR_OK; }

Error InitDriver();

Error SetCopies(Gstate, int count);
Error SetDisplayMode(Gstate, int mode);
Error WindowMove(Gstate, int dx, int dy);
Error BufferUpdate(Gstate);
Error ColourAdjust(Gstate, pt *);
Error GetFullPageBbox(Gstate, Bbox *box);
Error DefaultDevice(Gstate, char *device);

#define InitDriver		nothing
#define WindowMove		nothing
#define ColourAdjust	nothing
#define SetDisplayMode	nimp
#define GetFullPageBbox nimp
#define DefaultDevice	nimp
#define SetCopies		nimp
#define BufferUpdate	(Error (*)(Gstate))nimp
#define BufferSave		(Error (*)(Gstate))nimp
#define BufferRestore	(Error (*)(Gstate))nimp
#define Handle			(Error (*)(int,...))nimp

DRIVdriver init2 = {
	NULL, PF_PRINTERDRIVER,
	"G3 'normal' driver, for Qfax", 	/* name */
	204, 98,					/* xdpi, ydpi */
	{                           /* mono bitmap */
		1, 1,					/* 1 plane, 1 pixel deep */
		BITMAP_NO_BYTEGROUPING, /* no byte grouping */
		BITMAP_NO_PLANES,		/* no planes */
	},

	InitDriver,
	Define,

	InitInit,
	BufferSize,
	InitDone,
	CleanUp,

	InitPage,
	ShowPage,
	StopPage,

	SetCopies,
	SetDisplayMode,
	WindowMove,

	BufferUpdate,
	BufferSave,
	BufferRestore,

	PF_COLOURSPACE_GRAYSHADE,
	{
		ColourAdjust,
		ColourNearest,
		ColourDelta
	},

	GetFullPageBbox,
	DefaultDevice,

	Handle
};

DRIVdriver init = {
	&init2, PF_PRINTERDRIVER,
	"G3 'fine' driver, for Qfax",	  /* name */
	204, 196,					/* xdpi, ydpi */
	{                           /* mono bitmap */
		1, 1,					/* 1 plane, 1 pixel deep */
		BITMAP_NO_BYTEGROUPING, /* no byte grouping */
		BITMAP_NO_PLANES,		/* no planes */
	},

	InitDriver,
	Define,

	InitInit,
	BufferSize,
	InitDone,
	CleanUp,

	InitPage,
	ShowPage,
	StopPage,

	SetCopies,
	SetDisplayMode,
	WindowMove,

	BufferUpdate,
	BufferSave,
	BufferRestore,

	PF_COLOURSPACE_GRAYSHADE,
	{
		ColourAdjust,
		ColourNearest,
		ColourDelta
	},

	GetFullPageBbox,
	DefaultDevice,

	Handle
};
